/* 	DecoderFactory.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 5-aug-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Aug 14, 2007	dennis@brains2b.org		Brought the encoders within the Factory		
*/
package org.brains2b.coffee.decode;

import org.brains2b.coffee.CoffeeSpill;
import org.brains2b.coffee.message.Article;


/** Factory for creating Decoders and Encoders based
 *  on the content of an {@link Article} or on the name
 *  of the encoding
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 1.00 [14-08-2007]
 */
public class DecoderFactory {
	
	/**
	 * create the appropriate Decoder on the basis
	 * of the Article to decode.
	 * <p>The supplied article is added to the Decoder
	 * with {@link Decoder#decodePiece(String)} before 
	 * returning the Decoder
	 * 
	 * @param a Article, the Article which needs to be decoded
	 * @return Decoder
	 * @throws Exception, thrown if the {@link Decoder#decodePiece(String)} 
	 * was not succesfull.
	 */
    public static final Decoder createDecoder(Article a) throws Exception{
        Decoder decoder=null;
        int idx=0;
        final String content=a.getBody().getContent();
        final String mimeType=a.getHeader().getMimeType();
        if (content.indexOf("=ybegin")>-1) {
            decoder=new YencDecoder();
        } else if ((idx=content.indexOf("begin"))>-1 && Character.isDigit(content.charAt(idx+6)) && Character.isDigit(content.charAt(idx+7)) && Character.isDigit(content.charAt(idx+8))) {
            decoder=new UUDecoder();
        } else if (content.indexOf("MIME") >-1 && content.indexOf("Content-Type") >-1) {
            decoder=new MimeDecoder();
            idx=mimeType.indexOf("boundary");
            if (idx!=-1) {
                ((MimeDecoder) decoder).setBoundary(mimeType.substring(idx+10,mimeType.length()-1));
            }
        } else if (mimeType.startsWith("application/octet-stream")) {
            decoder=new Base64Decoder(mimeType.substring(mimeType.indexOf("name=")+5));
        }
        decoder.decodePiece(content);
        return decoder;
    }
    
    /**
     * create an Encoder for the given encoding
     * @param encoding String, case insensative name for the encoding, must
     * be one of the values in {@link #getEncoders()}
     * @return Encoder
     * @throws CoffeeSpill, thrown if the encoding supplied is not supported
     */
    public final static Encoder createEncoder(String encoding) throws CoffeeSpill {
    	if ("yenc".equalsIgnoreCase(encoding)) {
    		return new YencDecoder(); 
    	} else if ("uuencode".equalsIgnoreCase(encoding)) {
    		return new UUDecoder(); 
    	} else {
    		throw new CoffeeSpill("Encoder for " + encoding + " is not supported");
    	}
    }
    
    /**
     * get the Encoders supported
     * @return String[] names for the different encodings 
     */
    public final static String[] getEncoders() {
    	return new String[] {"yEnc","UUEncode" };
    }
}
