/* 	ConfigurationHandler.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 13-jan-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.file;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.brains2b.coffee.CoffeeSpill;
import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.TicketHandler;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.filter.FilterList;
import org.brains2b.coffee.message.local.CoffeeMenu;
import org.brains2b.coffee.message.local.Identity;
import org.brains2b.coffee.message.local.Options;
import org.brains2b.coffee.message.local.ServerDescriptor;
import org.brains2b.coffee.message.local.Subscription;
import org.brains2b.coffee.monitor.ProgressMonitor;

public class ConfigurationHandler implements TicketHandler,CoffeeExchange {

    private CoffeeMenu m_subscriptionList;
    private CoffeeMenu m_serverList;
    private CoffeeMenu m_identity;
    private CoffeeMenu m_options;
    
    private List m_monitors;
    
    private static final int[] EVENTS = new int[] {
        CoffeeEvent.GET_LIST,
        CoffeeEvent.LIST_ADD,
        CoffeeEvent.LIST_REMOVE,
        CoffeeEvent.SUBSCRIPTION_GROUP
        
    };
    
    public void addProgressMonitor(ProgressMonitor m) {
        if (m_monitors==null) {
        	m_monitors = new ArrayList();
        }
        m_monitors.add(m);
    }

    public void close() throws Exception {
       fireCoffeeEvent(this,CoffeeEvent.XML_SAVE_CONFIG,m_serverList);
       fireCoffeeEvent(this,CoffeeEvent.XML_SAVE_CONFIG,m_subscriptionList);
	   fireCoffeeEvent(this,CoffeeEvent.XML_SAVE_CONFIG,m_identity);
	   fireCoffeeEvent(this,CoffeeEvent.XML_SAVE_CONFIG,m_options);
	   fireCoffeeEvent(this, CoffeeEvent.XML_SAVE, FilterList.getInstance());
    }

    public int[] getHandledEvents() {
        return EVENTS;
    }

    public NNTPMessage handleEvent(CoffeeEvent event) throws CoffeeSpill {
        switch (event.getID()) {
            case CoffeeEvent.LIST_ADD:
                if (event.getArgument() instanceof ServerDescriptor) {
                    m_serverList.add((NNTPMessage) event.getArgument());
                } else if (event.getArgument() instanceof Subscription){
                    m_subscriptionList.add((NNTPMessage) event.getArgument());
                } else if (event.getArgument() instanceof Identity) {
                	m_identity.add((NNTPMessage) event.getArgument());
                } else  {
                	m_options.add((NNTPMessage) event.getArgument());
                }
                return null;
            case CoffeeEvent.LIST_REMOVE:
                if (event.getArgument() instanceof ServerDescriptor) {
                    m_serverList.remove((ServerDescriptor) event.getArgument());
                } else if (event.getArgument() instanceof Subscription){
                    m_subscriptionList.remove((Subscription) event.getArgument());
                } else {
                	m_identity.remove((NNTPMessage) event.getArgument());
                }
                return null;
            case CoffeeEvent.GET_LIST:
                if (m_serverList.getId().equals(event.getArgument())){
                    return m_serverList;
                } else if (m_subscriptionList.getId().equals(event.getArgument())){
                    return m_subscriptionList;
                } else if (m_identity.getId().equals(event.getArgument())) {
                	return m_identity;
                } else if (((String) event.getArgument()).startsWith(m_options.getId())) {
                	String arg = (String) event.getArgument();
                	arg = arg.substring(arg.indexOf("/")+1);
                	for (Iterator it = m_options.getList().iterator();it.hasNext();) {
                		NNTPMessage msg = (NNTPMessage) it.next();
                		if (msg.getId().equals(arg)) {
                			return msg;
                		}
                	}
                	return null;
                }
                throw new CoffeeSpill("Cannot find list for " + event.getArgument());
            case CoffeeEvent.SUBSCRIPTION_GROUP:
                Subscription sub = (Subscription) event.getArgument();
                ServerDescriptor sd = findServer(sub.getServer());
                fireCoffeeEvent(this, CoffeeEvent.CHANGE_SERVER, sd);
                fireCoffeeEvent(event.getExchange(), CoffeeEvent.NEWS_GROUP, sub.getName());
                return null;
            default: 
                return null;
        }
    }

    public boolean isAvailable() {
       return true;
    }

    public void open() throws Exception {
       m_serverList= new CoffeeMenu("server",ServerDescriptor.class);
       m_subscriptionList= new CoffeeMenu("subscription",Subscription.class);
       m_identity = new CoffeeMenu("identity",Identity.class);
       m_options  = new CoffeeMenu("options",Options.class);
       fireCoffeeEvent(this,CoffeeEvent.XML_READ_CONFIG,m_serverList);
       fireCoffeeEvent(this,CoffeeEvent.XML_READ_CONFIG,m_subscriptionList);
       fireCoffeeEvent(this,CoffeeEvent.XML_READ_CONFIG,m_identity);
       fireCoffeeEvent(this,CoffeeEvent.XML_READ_CONFIG,m_options);
 	   fireCoffeeEvent(this, CoffeeEvent.XML_READ, FilterList.getInstance());
    }

    public void removeProgressMonitor(ProgressMonitor m) {
        if (m_monitors!=null) {
        	m_monitors.remove(m);
        }

    }

    public void setCurrentMessage(NNTPMessage message) {
        if ("server".equals(message.getId())) {
            m_serverList= (CoffeeMenu) message;
        } else if ("subscription".equals(message.getId())) {
            m_subscriptionList= (CoffeeMenu) message;
        }
        
    }

    private void fireCoffeeEvent(CoffeeExchange source, int action, Object argument) {
        TicketBroker.getBroker().broker(new CoffeeEvent(source,action,argument));
    }
    
    private ServerDescriptor findServer(String name) {
        if (name==null || m_serverList==null) return null;
        for (Iterator it = m_serverList.getList().iterator();it.hasNext();) {
            ServerDescriptor sd = (ServerDescriptor) it.next();
        	if (name.equals(sd.getId())) return sd;
        }
        return null;
    }
}
