/* 	ArticleTreeTableModel.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 2-jan-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.gui.table;

import java.util.Collections;
import java.util.Comparator;

import javax.swing.JTree;
import javax.swing.event.EventListenerList;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.ArticleHeader;
import org.brains2b.coffee.util.CfDateHelper;
import org.brains2b.coffee.util.CfSizeHelper;

public class ArticleTreeTableModel extends AbstractTableModel implements
        TreeModel {

    public class TreeHandler implements TreeExpansionListener {

        public void treeCollapsed(TreeExpansionEvent event) {
            fireTableDataChanged();
            
        }

        public void treeExpanded(TreeExpansionEvent event) {
             fireTableDataChanged();
        }
        
    }
    
    class ArticleComparator implements Comparator {
        private int m_sortIndex;
        
        public ArticleComparator(int sortIndex) {
            m_sortIndex=sortIndex;
        }
        
        /* (non-Javadoc)
         * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
         */
        public int compare(Object o1, Object o2) {
            int result=-1;
            if (o1 instanceof ArticleNode && o2 instanceof ArticleNode) {
                Article a1=((ArticleNode) o1).getArticle();
                Article a2=((ArticleNode) o2).getArticle();
                int idx=Math.abs(m_sortIndex);
                switch (idx) {
                	case 1:
                	    result=a1.getHeader().getSubject().compareTo(a2.getHeader().getSubject());
                	    break;
                	case 2:
                	    result=a1.getHeader().getAuthor().compareTo(a2.getHeader().getAuthor());
                	    break;
                	case 3:
                		if (a2.getHeader().getDate()==null) result= -1;
                		else result=a1.getHeader().getDate().compareTo(a2.getHeader().getDate());
                	    break;
                	case 4:
                	    result=a1.getHeader().getSize()-a2.getHeader().getSize();
                	    break;
                }
            }
            return (m_sortIndex<0?result*-1:result);
        }
    }

    private ArticleNode m_root;
    private EventListenerList m_listeners;
    private JTree m_tree;
    private int m_sortIndex;
    
    public ArticleTreeTableModel() {
        super();
        m_listeners= new EventListenerList();
    }
    
    public void setTree(JTree tree) {
        m_tree=tree;
        m_tree.addTreeExpansionListener(new TreeHandler());
    }
    
    public void setData(ArticleNode node) {
        m_root=node;
        m_tree.setModel(this);
        sort(2,true); // Sort the list on date
    }
    
    public Object getChild(Object parent, int index) {
       return ((ArticleNode)parent).getChildAt(index);
    }

    public int getChildCount(Object parent) {
        return ((ArticleNode)parent).getChildCount();
    }

    public int getIndexOfChild(Object parent, Object child) {
         return ((ArticleNode)parent).getIndex((ArticleNode) child);
    }

    public Object getRoot() {
        if (m_root==null || m_root.isLeaf()) {
            return null;
        }
       return m_root;
    }

    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return columnIndex==0;
    }
    public boolean isLeaf(Object node) {
        return getChildCount(node)==0;
    }

    public void addTreeModelListener(TreeModelListener l) {
        m_listeners.add(TreeModelListener.class, l);
    }

    public void removeTreeModelListener(TreeModelListener l) {
        m_listeners.remove(TreeModelListener.class, l);
    }
    
    protected void fireTreeDataChanged(TreePath path) {
        if (path==null) {
        	path=m_tree.getPathForRow(0);
        	
        }
        TreeModelEvent tme=new TreeModelEvent(this,path);
        TreeModelListener[] listeners= (TreeModelListener[]) m_listeners.getListeners(TreeModelListener.class);
        for (int i=0;i<listeners.length;i++) {
            listeners[i].treeStructureChanged(tme);
        }
        fireTableDataChanged();
    }

    public void valueForPathChanged(TreePath path, Object newValue) {
        }

    public int getColumnCount() {
        return 5;
    }
    
    public Class getColumnClass(int columnIndex) {
        if (columnIndex==0) {
            return ArticleNode.class;
        } else {
            return String.class;
        }
        //return super.getColumnClass(columnIndex);
    }

    public int getRowCount() {
        if (m_tree==null) {
            return 0;
        }
        return m_tree.getRowCount();
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
    	final TreePath path = m_tree.getPathForRow(rowIndex);
    	if (path==null) return null;
        final Article a = ((ArticleNode) path.getLastPathComponent()).getArticle();
        if (a==null) return null;
        
        ArticleHeader ah= a.getHeader();
        Object result=null;
        if (a!=null) {
            switch (columnIndex) {
               case 0:
                    result=ah.getSubject();
                    break;
                case 1:
                    result=ah.getAuthor();
                    break;
                case 2:
                    result=CfDateHelper.format(ah.getDate());
                    break;
                case 3:
                    result=CfSizeHelper.getHumanReadable(ah.getSize());
                    break;
                case 4:
                    result=String.valueOf(a.getTotalParts());
                    break;
            }
        }
        return result;
    }

    public Article getArticle(int row) {
        if (row==-1) {
            return null;
        }
        TreePath path = m_tree.getPathForRow(row);
        if (path==null) {
            return null;
        }
        Article a = ((ArticleNode) path.getLastPathComponent()).getArticle();
        return a;
    }
    
    public void sort(int columnIndex, boolean descending) {
        if (m_root.getChildCount()==0) {
        	fireTreeDataChanged(null);
            return;
        }
        int oldIndex = m_sortIndex;
        m_sortIndex = (columnIndex+1);
        if (descending && m_sortIndex==Math.abs(oldIndex)) {
        	m_sortIndex=m_sortIndex*-1;
        } else if (m_sortIndex==Math.abs(oldIndex)) {
        	m_sortIndex=oldIndex*-1;
        }
        Collections.sort(m_root.getArticles(),new ArticleComparator(m_sortIndex));
        fireTreeDataChanged(null);
    }

	public int getSortIndex() {
		return m_sortIndex;
	}

	public void setSortIndex(int sortIndex) {
		m_sortIndex = sortIndex;
	}
}
