/* 	QueryMessage.java
 *	Copyright (c) 2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Feb 25, 2008
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.plugin.search;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.NewsGroup;
import org.brains2b.coffee.message.filter.MessageFilter;
import org.brains2b.coffee.plugin.MimeType;
import org.brains2b.saxo.SAXWriter;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;
/** Local Message for storing Query definitions of NewsSearch
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version $Revision: 1.3 $ $Date: 2010/07/05 01:18:10 $
 */
public class QueryMessage implements NNTPMessage, MimeType {
	
	public final static int QUERY = 19032;
	
	private final static String m_mimeType = "text/xml.query";
	
	private boolean m_direct;
	private int m_frequency;
	private String m_id;
	private List m_filters;
	private List m_newsgroups;
	private List m_articles;
	
	/**
	 * Constructor
	 *
	 */
	public QueryMessage() {
		super();
		m_newsgroups = new ArrayList();
		m_filters = new ArrayList();
	}
	
	public String getId() {
		return m_id;
	}

	public int getSize() {
		return m_articles==null?0:m_articles.size();
	}

	public int getType() {
		return QUERY;
	}

	public boolean hasContent() {
		return m_articles!=null;
	}

	public boolean isHeaderSet() {
		return false;
	}

	public void writeMessage(SAXWriter sw) throws SAXException {
		AttributesImpl atts = new AttributesImpl();
		atts.addAttribute(null, null, "id", "CDATA", m_id);
		atts.addAttribute(null, null, "frequency", "CDATA", String.valueOf(m_frequency));
		sw.startElement("querydef", atts);
		if (m_filters!=null) {
			for (Iterator it = m_filters.iterator();it.hasNext();) {
				MessageFilter mf = (MessageFilter) it.next();
				mf.writeMessage(sw);
			}
		}
		
		if (m_newsgroups!=null) {
			for (Iterator it = m_newsgroups.iterator();it.hasNext();) {
				NewsGroup ng = (NewsGroup) it.next();
				ng.writeList(sw);
			}
		}
		
		if (m_articles!=null) {
			for (Iterator it = m_articles.iterator();it.hasNext();) {
				Article a = (Article) it.next();
				a.writeList(sw);
			}
		}
		
		sw.endElement("querydef");
		
	}

	/**
	 * get a List of Filters making up this query
	 * @return List, list of {@link MessageFilter}
	 */
	public List getFilters() {
		return m_filters;
	}

	/**
	 * set a List of Filters making up this query
	 * @param filters List, list of {@link MessageFilter}
	 */
	public void setFilters(List filters) {
		m_filters = filters;
	}
	
	/**
	 * add a Filter to this Query
	 * @param filter MessageFilter
	 */
	public void addFilter(MessageFilter filter) {
		if (m_filters==null) m_filters = new ArrayList();
		m_filters.add(filter);
	}

	/**
	 * get a List of Newsgroups this Query should search in
	 * @return List, list of {@link NewsGroup}
	 */
	public List getNewsgroups() {
		return m_newsgroups;
	}

	/**
	 * set a List of Newsgroups this Query should search in
	 * @param newsgroups List, list of {@link NewsGroup}
	 */
	public void setNewsgroups(List newsgroups) {
		m_newsgroups = newsgroups;
	}
	
	public void addNewsGroup(NewsGroup group) {
		if (m_newsgroups==null) m_newsgroups = new ArrayList();
		m_newsgroups.add(group);
	}

	public void setId(String id) {
		m_id = id;
	}

	public String toString() {
		return m_id;
	}

	/** add an Article which was found as a result of this Query
	 * @param a Article
	 */
	public void addArticle(Article a) {
		if (m_articles==null) m_articles = new ArrayList();
		m_articles.add(a);
	}
	
	/**
	 * get a List of Articles which were found by this Query
	 * @return List, list of {@link Article}
	 */
	public List<Article> getArticles() {
		return m_articles;
	}

	/**
	 * set a List of Articles as result for this Query
	 * @param articles List, list of {@link Article}
	 */
	public void setArticles(List articles) {
		m_articles = articles;
	}
	
	public String getMimeType() {
		return m_mimeType;
	}

	public int getFrequency() {
		return m_frequency;
	}

	public void setFrequency(int frequency) {
		m_frequency = frequency;
	}

	boolean isDirect() {
		return m_direct;
	}

	void setDirect(boolean direct) {
		m_direct = direct;
	}
	
	public void addHeader(int id, String s) {
	}

	public void addLine(String s) {
	}

	public void endMessage() {
	}

	public int getNewMessages() {
		if (getArticles()==null) return 0;
		int newcnt = 0;
		for (Article a:getArticles()) {
			newcnt += a.isNew() ? 1 : 0;
		}
		return newcnt;
	}
	
}
