/* 	SubjectHelper.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 3-aug-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * 25-02-2008	dennis@brains2b.nl		Class should not be a singleton, Internalize SubjectInfo, optimized 		
*/
package org.brains2b.coffee.util;

import java.io.File;

import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.MultiPartGroup;
import org.brains2b.coffee.message.local.FileListMessage;
import org.brains2b.coffee.message.local.FileMessage;
import org.brains2b.coffee.message.local.Options;


/** Helper class for Article Subjects
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.89.1 [25-02-2008]
 */
public class SubjectHelper {
    
	public static class SubjectInfo {
		public int part;
		public int total;
		public String subject;
		public boolean isYenc;
		
	}
	
    private SubjectInfo m_info = new SubjectInfo();
    
	/**
	 * Split the Subject text into String, parts and totals for use in {@link MultiPartGroup}
	 * @param s String, the complete subject
	 * @return {@link SubjectInfo}
	 * @see MultiPartGroup
	 */
	public final SubjectInfo getTruncated(String s) {
        return getTruncated(s,s.length());
    }
    
	/**
	 * Split the Subject text into String, parts and totals for use in {@link MultiPartGroup}
	 * @param s String, the complete subject
	 * @param end int, the end of the first part of the String
	 * @return {@link SubjectInfo}
	 * @see MultiPartGroup
	 */
    private final SubjectInfo getTruncated(String s, int end) {
    	final String su =s.toUpperCase();
        if (su.startsWith("RE:")) {
            m_info.subject=null;
        	return m_info;
        }
        m_info.isYenc=su.contains("YENC");
        
        final char[] c = s.toCharArray();
        int idx=-1;
        int div=-1;
        int last=-1;
        for (int i=end-1;i>=0;i--) {
            if (c[i]==')') {
                last=i;
            } else if (c[i]=='/' || c[i]=='\'') {
                div=i;
            } else if (c[i]=='(') { idx=i; break;}
        }
        if (idx!=-1 && last!=-1 && div!=-1 && idx<last && div<last && div>idx) {
    	    try {
    	        final int part=Integer.parseInt(s.substring(idx+1,div).trim());
    	        final int total=Integer.parseInt(s.substring(div+1,last).trim());
    	        if (part==1 && part==total) {
                     m_info.subject=null;
        			return m_info;
                }
    	        
    	        while (idx>0 && c[idx-1]==' ') {
                    idx--;
                }
                
    	        last=last+1;
                //System.arraycopy(c, 0, c,0, idx);
                if (last<c.length) {
                    System.arraycopy(c,last,c,idx,c.length-last);
                }
                
    	        m_info.subject = String.valueOf(c,0,idx + (last<c.length?c.length-last:0));
                m_info.part = part;
                m_info.total = total;
                return m_info;
    	    } catch (NumberFormatException nfex) {
    	        return getTruncated(s,idx);
    	    }
        } else if (idx!=-1) {
            return getTruncated(s,idx);
        }
        m_info.subject=null;
        return m_info;
    }
    
    /**
     * create the subject for an Article with attachment being posted
     * @param a Article, the article to Post
     * @param o Options, the Options for posting messages
     * @param file int, the file in a collection of files
     * @param totalFile int , the total number of files
     * @return String, the formatted Subject
     */
    public final static String createSubject(Article a, Options o, int file, int totalFile) {
    	if (a==null) return null;
		StringBuffer sb = new StringBuffer();
		sb.append(a.getSubject());
		FileMessage fm = a.getFile();
		
		if (fm instanceof FileListMessage) {
			fm = new FileMessage();
			fm.setFile((File) ((FileListMessage) a.getFile()).getFiles().get(file-1));
		} else {
			fm = a.getFile();
		}
		
		if (fm==null) {
			return sb.toString();
		} 
		if (o.getBoolean("show-file-count") && totalFile>1) {
			sb.append(" [").append(file).append("/").append(totalFile).append("]");
		}
		if (o.getBoolean("show-filename")) {
			sb.append(" ").append(fm.getFile().getName());
		} 
		if ("yEnc".equals(o.getString("encoding"))) {
			sb.append(" yEnc");
		}
		if (a.getPart()==0) {
			long block = Long.parseLong(o.getString("block-size"))*1024l;
			int tb = (int) (fm.getFile().length()/block+1);
			if (tb>1) {
				sb.append(" (").append(a.getPart()+1).append("/").append(tb).append(")");
			}
		} else if (!(a.getTotalParts()==1 && a.getPart()==1)) {
			sb.append(" (").append(a.getPart()).append("/").append(a.getTotalParts()).append(")");
		}
		
 		return sb.toString();
    }
}
